/**
 * @desc More or less comfortable abstraction of set<T>.
 */
#ifndef _GENERATESET_H
#define _GENERATESET_H

#include <set>
#include <vector>
#include <algorithm>
using namespace std;

template<class T>
class GenerateSet
{
public:
    set<T> data;

    GenerateSet() = default;

    GenerateSet(const T &item)
    {
        data.insert(item);
    }

    GenerateSet(const GenerateSet &) = default;
    GenerateSet(GenerateSet&&) = default;

    GenerateSet &insert(const T &item)
    {
        data.insert(item);
        return *this;
    }

    GenerateSet &erase(T item)
    {
        data.erase(item);
        return *this;
    }

    GenerateSet &merge(GenerateSet &&s)
    {
        for (auto item : s)
            add(item);
        return *this;
    }

    GenerateSet &merge(const GenerateSet &s)
    {
        for (auto item : s)
            insert(item);
        return *this;
    }

    GenerateSet &merge(const set<T> &s)
    {
        for (auto item : s)
            add(item);
        return *this;
    }

    GenerateSet &operator<<(T item)
    {
        return insert(item);
    }

    GenerateSet &operator<<(const GenerateSet &s)
    {
        return merge(s);
    }

    GenerateSet &operator<<(GenerateSet &&s)
    {
        return merge(std::move(s));
    }

    bool contains(T item)
    {
        return data.find(item) != data.end();
    }

    GenerateSet &&intersection(const GenerateSet &s) const
    {
        GenerateSet i;
        for (auto item : s)
            if (contains(item))
                i.add(item);
        return std::move(i);
    }

    bool empty()
    {
        return data.empty();
    }

    int size()
    {
        return data.size();
    }

    void clear()
    {
        data.clear();
    }

    void get_sorted_list(vector<T> &sorted)
    {
        sorted.clear();
        for (auto item : data)
            sorted.push_back(item);
        std::sort(sorted.begin(), sorted.end());
    }

    using iterator = typename set<T>::iterator;
    using const_iterator = typename set<T>::const_iterator;
    iterator begin() noexcept { return data.begin(); }
    iterator begin() const noexcept { return data.begin(); }
    const_iterator cbegin() noexcept { return data.begin(); }
    const_iterator cbegin() const noexcept { return data.begin(); }
    iterator end() { return data.end(); }
    iterator end() const { return data.end(); }
    const_iterator cend() const noexcept { return data.end(); }

};

#endif
