/**
 * @desc RTL query language
 */
#include "query.h"
#include "output.h"
#include "querylanguage.h"

namespace vam
{
namespace rtl
{

string querycmd_to_str(QueryType type)
{
    for (int i = 0; qtokens[i].type != T_EOF; i++)
        if (qtokens[i].type == type)
            return qtokens[i].value;
    if (type == Q_RTLOBJECT)
        return "<RTLObject>";
    else
        return "<unknown-command>";
}

string query_to_str(const Query &q)
{
    string out;
    int i;
    switch (q.type)
    {
        case Q_CONST:
        case Q_VAR:
            out = q.value;
            break;
        case Q_NONE:
            out = "<none-object>";
            break;
        case Q_AT:
            out = "(" + querycmd_to_str(q.type) + ' ' +
                query_to_str(*q.subqueries[0]) + ' ' +
                std::to_string(q.aux_int) + ')';
            break;
        case Q_ATINC:
            out = "(" + querycmd_to_str(q.type) + ' ' + 
                q.value + ' ' +
                query_to_str(*q.subqueries[0]) + " #" +
                std::to_string(q.aux_int) + ')';
            break;
        case Q_LOAD:
            out = "(" + querycmd_to_str(q.type) + " \"" + q.value + "\")";
            break;
        case Q_SET_WHERE:
        case Q_TUPLE_WHERE:
            out = "(" + querycmd_to_str(q.type) + ' ';
            out += query_to_str(*q.subqueries[0]) + " where";
            i = 0;
            for (auto &subq : q.subqueries)
            {
                if (!i++) continue;
                out += ' ';
                out += query_to_str(*subq);
            }
            out += ")";
            break;
        case Q_EXISTS:
        case Q_FORALL:
            out = "(" + querycmd_to_str(q.type) + ' ' + q.value;
            for (auto &subq : q.subqueries)
            {
                out += ' ';
                out += query_to_str(*subq);
            }
            out += ")";
            break;

        default:
            out = "(" + querycmd_to_str(q.type);
            for (auto &subq : q.subqueries)
            {
                out += ' ';
                out += query_to_str(*subq);
            }
            out += ")";
            break;
    }
    return out;
}

} // namespace rtl
} // namespace vam
