/**
 * @desc RTL manipulation query language
 */
#ifndef _RTLMANIP_QL_H
#define _RTLMANIP_QL_H

#include "vamrtl.h"
#include "input.h"
#include "query.h"
#include "rtlmanip.h"

using namespace std;

namespace vam
{
namespace rtl
{

extern const Token qtokens[];

class QueryParser : public RTLStringParser
{
public:
    QueryParser(System &s) : RTLStringParser(s) {}

    void load_command(const string &s);
    void load_script(const string &filename);
    Query parse_next_command();

private:
    Token unget_tok = Token{-1};
    /// unget previous token
    void unget_token(Token t)
    {
        unget_tok = t;
    }
    Token next_token(Meta *meta)
    {
        if (unget_tok.type != -1)
        {
            Token t = unget_tok;
            unget_tok.type = -1;
            return t;
        }
        else
            return RTLStringParser::next_token(meta);
    }

    Query p_query(bool accept_rpar = false);
    Query p_str(QueryType type);
    Query p_select();
    Query p_let();
    Query p_undef();
    Query p_debug_variable(QueryType type);
    Query p_debug_varsdep();
    Query p_system_shell();
    Query p_set(QueryType type);
    Query p_fol(QueryType type);
    Query p_0ary(QueryType type);
    Query p_unary(QueryType type);
    Query p_binary(QueryType type);
    Query p_ternary(QueryType type);
    Query p_nary_operation(QueryType type);
    void p_nary_subqueries(Query &q);
    Query p_at();
    Query p_in();
    Query p_if();
    Query p_isinstance();
    Query p_exprtype();
    Query p_path(QueryType type);
    Query p_coi(QueryType type);
    Query p_expr_coi(QueryType type);
    Query p_smtlib_binary(QueryType type);
    Query p_smtlib_ternary(QueryType type);
    /// \{
    /// Queries useful for scripting
    Query p_loadsave_file(QueryType type);
    /// \}
};

} // namespace rtl
} // namespace vam

#endif
