/**
 * VariableGroup class for interpret of RTL queries.
 */
#ifndef _QUERYVARGROUP_H
#define _QUERYVARGROUP_H

#include "query.h"
#include "queryvariableinfo.h"
#include <set>

using namespace std;

namespace vam
{
namespace rtl
{

class VariableGroup
{
public:
    // dependency-ordered list of variables
    vector<PVariableInfo> variables;

    // bound-variable driver
    PQuery driver = nullptr;

    VariableGroup() = default;
    VariableGroup(PQuery driver) : driver(driver) {}
    //VariableGroup(PQuery driver, PVariableInfo v) : driver(driver)
    VariableGroup(PQuery driver, VarSet &vars) : driver(driver)
    {
        insert(vars);
    }

    /**
     * @brief   Inserts a new variable to variable group at position given by
     * its bound_prerequisites.
     */
//    void insert(PVariableInfo newv);

    /**
     * @brief   Initialize vgroup from the given set of variables.
     */
    void insert(VarSet &vars);

    /**
     * @brief   Generate a new combination of values of group of variables.
     * @return  true if generated some valid combination of values
     */
    bool next_value_combination();

    /**
     * @brief   Sets the driver for undriven variables.
     */
    void drive_undriven();

    /**
     * @brief   Resets value of all variables in the group.
     */
    void reset()
    {
        for (auto v : variables)
            v->reset();
    }

};

} // namespace rtl
} // namespace vam

#endif
