#ifndef _SWITCH_PARSER_MACRO_H
#define _SWITCH_PARSER_MACRO_H

#include "vam_err.h"

#define PASSERT(c, t, msg) \
    ASSERT(c, t.loc() << msg)

#define TOKINIT(meta) Token t; Meta *_tok_meta = meta

#define TOKDECL Token t; Meta *_tok_meta = NULL

#define TOKNEW \
    t = next_token(_tok_meta); \
    PASSERT(t.type != T_EOF, t, "unexpected end-of-file"); \
    PASSERT(t.type != T_UNDEF, t, "undefined token")

#define TOKRNEW \
    TOKNEW; \
    t = tok2keyword(t)

#define TOKQNEW \
    TOKNEW; \
    t = tok2querykeyword(t)

#define PEXPECT(tt, msg) \
    PASSERT(t.type == tt, t, \
            "expected " << msg << " before '" << t.value << "'")

#define REXPECT(tt, msg) \
    t = tok2keyword(t); \
    PASSERT(t.type == tt, t, \
            "expected " << msg << " before '" << t.value << "'")

#define CHECKWIDTH(expr) \
    try { expr->reset_bitwidth(); } \
    catch (VAMException e) { RTLTHROWLOC(t.loc(), e.msg); }

#else

#undef PASSERT
#undef TOKINIT
#undef TOKNEW
#undef TOKRNEW
#undef PEXPECT
#undef REXPECT
#undef CHECKWIDTH

#endif
