/**
 * @brief Support of exceptions and logging in VAM operations.
 */
#ifndef _VAM_ERR_H
#define _VAM_ERR_H

#include <exception>
#include <iostream>
#include <sstream>
#include <string>

using namespace std;

namespace vam
{

#define WARNING(warn) \
    (cerr << "VAM warning: " << __FILE__ << ":" << __LINE__ << ": " << warn << endl)

#define THROW(msg) \
    { ostringstream _throw_ss; \
      _throw_ss << __FILE__ << ":" << __LINE__ << ": " << msg; \
      throw VAMException(_throw_ss.str()); }

/*
#define THROW(msg) \
    { ostringstream _throw_ss; \
      _throw_ss << msg; \
      throw VAMException(_throw_ss.str()); }
#define WARNING(warn) \
    (cerr << "VAM warning: " << warn << endl)
*/

#define ASSERT(cond,msg) \
    { if (!(cond)) THROW(msg) }

class VAMException : public exception
{
private:
    VAMException() {}
public:
    string msg;

    VAMException(string m) noexcept : msg(m) { }

    const char* what() const noexcept { return "VAMException"; }

    void print()
    {
        cerr << what() << ": " << msg << endl;
    }

};

}

#endif
