/**
 * @brief   High-level test suite for RTL Queries
 */

#define BOOST_TEST_DYN_LINK
#include <boost/test/unit_test.hpp>
#include "vaml_test.h"
#include "query.h"
#include "querylanguage.h"

BOOST_AUTO_TEST_SUITE(rtl_queries)

BOOST_AUTO_TEST_CASE(references)
{
    // setup
    string input =
            "(model modelname\n"
            "  (pipeline-stages xx)\n"
            "  (sig v1 1 (vt control) (connections (r0 we) ) )\n"
            "  (sig v3 1 (vt data ) (connections (r0 q) ) )\n"
            "  (sig v4 1 (vt unknown) )\n"
            "  (sig v5 1 (vt unknown) (connections (r0 d) ) )\n"
            "  (reg r0 1 (we v1) (d v5) (q v3 ) )\n"
            "  (fnode f0 (input v3 v4 v5 ) (output v1 v4 ) (stage xx) (assign \n"
            "(:= v1 (|| v5 v4 ) ) \n"
            "(:= v4 (&& v3 v5 ) ) \n"
            ")) \n"
            ")\n";
    string query_command = 
        "(references (outputs (select-signal \"v5\")))";

    RTLParser p;
    System system;
    p.parse_str(input, &system);
    rtl::QueryParser qparser{system, query_command};
    rtl::Query query;
    try { query = qparser.parse_next_command(); }
    catch(VAMException &e) { BOOST_ERROR("error parsing " << query_command); }
   
    BOOST_CHECK_EQUAL(query.type, rtl::Q_REFERENCES); 
    rtl::PQuery query_to_execute = make_shared<rtl::Query>(query);
    rtl::Interpret interpret{system};
    interpret.exec(query_to_execute);
    
}

BOOST_AUTO_TEST_SUITE_END()
